﻿/*! \file Controller.cs
 * \brief Handles import, presets, and provides an interface to the volume object.
 */

using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using UnityEditor;
using System.IO;

[System.Serializable]
public struct Preset
{
    public string name;
    public Texture3D dataTex;
    public Texture2D transferTex;
    public float min, max;
    public Vector3 positionOffset;
    public Vector3 rotationOffset;
}

/// <summary>
/// Controller for volume object.
/// </summary>
public class Controller : MonoBehaviour
{

    public GameObject target;
    public RawImage histogram;
    public bool saveTexture = false;
    public int numBits = 8;
    public int dimX;
    public int dimY;
    public int dimZ;

    public Preset[] presets;


    /// <summary>
    /// Imports the data file and saves it as a texture.
    /// </summary>
    /// <param name = "file">The data file being imported. </param>
    public void Import(string file)
    {
        var importer = new Importer();
        Importer.ImportResult result = importer.Import(file, numBits, dimX, dimY, dimZ);

        if (saveTexture)
        {
            // This caused errors on build as AssetDatabase is only available in the editor.

            //var fileName = Path.GetFileNameWithoutExtension(file);
            //AssetDatabase.CreateAsset(result.volume, "Assets/DataTextures/" + fileName + ".asset");
            //AssetDatabase.CreateAsset(result.histogram, "Assets/DataTextures/" + fileName + "-histogram.asset");
        }
    }

    /// <summary>
    /// Setting preset data file as texture from user input through GUI
    /// </summary>
    /// <param name = "index">Index in a list of available presets </param>
    public void SelectPreset(int index)
    {
        var preset = presets[index];

        var transform = target.GetComponent<Transform>();
        transform.position = preset.positionOffset;
        transform.rotation = Quaternion.Euler(preset.rotationOffset);

        var meshRenderer = target.GetComponent<MeshRenderer>();
        var material = meshRenderer.sharedMaterial;

        material.SetTexture("_VolumeTex", preset.dataTex);
        material.SetTexture("_TransferFunction", preset.transferTex);
        material.SetFloat("_Min", preset.min);
        material.SetFloat("_Max", preset.max);
    }

    /// <summary>
    /// Changing the lerp value for traditional/hue preesrving blending
    /// </summary>
    /// <param name = "value">The lerp value </param>
    public void SetLerp(float value)
    {
        var material = target.GetComponent<MeshRenderer>().sharedMaterial;
        material.SetFloat("_Lerp", value);
    }

    /// <summary>
    /// Sets amount of bias used.
    /// </summary>
    /// <param name = "value">Value from bias lerp </param>
    public void SetBias(float value)
    {
        var material = target.GetComponent<MeshRenderer>().sharedMaterial;
        material.SetFloat("_Bias", value);
    }

    /// <summary>
    /// Sets shading intensity
    /// </summary>
    /// <param name = "value">Value from shading lerp </param>
    public void SetShadingIntensity(float value)
    {
        var material = target.GetComponent<MeshRenderer>().sharedMaterial;
        material.SetFloat("_ShadingIntensity", value);
    }
}
