﻿/*! \file Importer.cs
 * \brief Processes data files.
 */

using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using System.IO;
using System;

/// <summary>
/// Performs the actual data import and returns a 3d texture.
/// </summary>
public class Importer
{

    public struct ImportResult
    {
        public Texture3D volume;
        //public Texture2D histogram;
    }

    /// <summary>
    /// Reads the data and processes it so it can be set as texture.
    /// </summary>
    /// <param name = "file">The data file being imported. </param>
    /// <param name = "numBits">Size of file. </param>
    /// <param name = "dimX">X dimension of data. </param>
    /// <param name = "dimY">Y dimension of data. </param>
    /// <param name = "dimZ">Z dimension of data. </param>
    /// <returns> A texture that can be used on the volume object.</returns>
    public ImportResult Import(string file, int numBits, int dimX, int dimY, int dimZ)
    {
        var extension = Path.GetExtension(file);
        FileStream fs = new FileStream(file, FileMode.Open);
        BinaryReader reader = new BinaryReader(fs);

        switch(extension)
        {
            case ".raw":
                break;

            case ".dat":
                numBits = 16;
                dimX = reader.ReadUInt16();
                dimY = reader.ReadUInt16();
                dimZ = reader.ReadUInt16();
                break;
        }

        var size = dimX * dimY * dimZ;

        float maxValue = 0;
        ushort[] data = new ushort[size];
        for (int i = 0; i < size; i++)
        {
            data[i] = numBits == 8 ? reader.ReadByte() : reader.ReadUInt16();
            if (data[i] > maxValue)
                maxValue = (float)data[i];
        }

        reader.Close();
        fs.Close();

#if false
        // TODO find a way to upload beetle data to a 3d-texture.
        // Can't simply use a color array -> this would be 5GB -> out of memory

        dimZ = 5;

        Texture3D texture = new Texture3D(dimX, dimY, dimZ, TextureFormat.RFloat, false);
        texture.wrapMode = TextureWrapMode.Clamp;

        Texture2DArray slices = new Texture2DArray(dimX, dimY, dimZ, TextureFormat.RFloat, false);

        for (int z = 0; z < dimZ; z++)
        {
            // Texture2D slice = new Texture2D(dimX, dimY, TextureFormat.RFloat, false, true);
            Color[] colors = new Color[dimX * dimY];

            for (int y = 0; y < dimY; y++)
            {
                for (int x = 0; x < dimX; x++)
                {
                    int index = x + y * dimX + z * (dimX * dimY);
                    float value = (float)data[index] / maxValue;

                    colors[x + y * dimX] = new Color(value, value, value, value);
                }
            }

            //slice.SetPixels(colors);
            //slice.Apply(); //, true);

            slices.SetPixels(colors, z);
            slices.Apply();

            // texture.Apply();
        }

        // somehow the 2d texture gets only copied into the 0th slice of the 3d texture
        for (int z = 0; z < dimZ; z++)
            Graphics.CopyTexture(slices, z, texture, 0);

#else
        
        Color[] colors = new Color[size];

        /*
        int histogramWidth = 256, histogramHeight = 100;
        int[] bins = new int[histogramWidth];

        for (int z = 0; z < dimZ; z++)
        {
            for (int y = 0; y < dimY; y++)
            {
                for (int x = 0; x < dimX; x++)
                {
                    int index = x + y * dimX + z * (dimX * dimY);
                    float value = (float)data[index] / maxValue;

                    int bin = (int)Math.Round(value * (histogramWidth - 1), 0);
                    bins[bin]++;

                    colors[index] = new Color(value, value, value, value);
                }
            }
        }

        int max = 0;
        for (int i = 0; i < histogramWidth; i++)
        {
            if (bins[i] > max)
                max = bins[i];
        }

        float scale = (float)histogramHeight / (float)max;

        Texture2D histogram = new Texture2D((int)histogramWidth, (int)histogramHeight, TextureFormat.RGBA32, false);
        Color[] initialColors = new Color[histogramWidth * histogramHeight];

        for (int i = 0; i < histogramWidth * histogramHeight; i++)
            initialColors[i] = new Color(1, 1, 1, 0.2f);

        histogram.SetPixels(0, 0, histogramWidth, histogramHeight, initialColors);

        for (int x = 1; x <= histogramWidth; x++)
        {
            uint val = (uint)Math.Round(scale * bins[x - 1]);
            for (int y = 1; y < val; y++)
            {
                histogram.SetPixel(x, y, Color.cyan);
            }
        }
        */

        Texture3D texture = new Texture3D(dimX, dimY, dimZ, TextureFormat.RFloat, false);
        texture.wrapMode = TextureWrapMode.Clamp;
        texture.SetPixels(colors);
        texture.Apply();

        ImportResult result = new ImportResult();
        result.volume = texture;
        //result.histogram = histogram;
        
        return result;
#endif
    }
}
