﻿/*! \mainpage Comparison of traditional and hue preserving color blending
 * \image html 04.png width=800px
 * \section intro_sec Introduction
 * This program aims to visualise the difference between traditional and hue preserving color blending described in Chuang et al. 
 * The comparison is done through a lerp functionality that flicks between the two blending modes, and the user can also compare different data sets.
 * The algorithm preserves the original hue of the blended colors and therefore gives a clearer visualisation of nominal data since the colors are more contrasted. 
 * A bias function is also implemented to avoid gray colors that come from blending different colors and instead use the more saturated color values.
 *
 * \section usage Usage
 * We predesigned a few datasets and their transfer textures for the user to explore. 
 * There are two kinds of bias functions:
 * 
 * 1. Lerp between traditional and hue-preserving blending. This kind of "bends" the regular blending curve towards hue-preserving blending.
 * 2. Bias only affects hue-preserving blending. This causes the color to stay more in the saturated part.
 * 
 * \section impl Implementation
 * Implementation is done in Unity3D via GPU-based front-to-back volume raycasting.
 * The most important files to look at are [Volume.shader](../src/Assets/Shaders/Volume.shader) and [functions.cginc](../src/Assets/Shaders/functions.cginc).
 * 
 * \section Links
 * * [Executable](../bin/bin.zip)
 * * [Source](../src/src.zip) (only script files)
 * * [Original paper](../hue-preserving-color-blending.pdf)
 */

/*! \file VolumeObject.cs
 * \brief Responsible for adding two extra passes to the pipeline for setting up ray start and end positions.
 */
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Rendering;

// [ExecuteInEditMode]
/// <summary>
/// A volume object with a set texture and color.
/// </summary>
public class VolumeObject : MonoBehaviour
{
    public Shader volumeRayDataShader;

    /// <summary>
    /// Called on app start.
    /// </summary>
    void Start()
    {
        AddBuffer();
    }

    /// <summary>
    /// Hacks the unity pipeline and adds two extra passes. 
    /// </summary>
    public void AddBuffer()
    {
        var cam = Camera.main;
        var meshRenderer = GetComponent<MeshRenderer>();
        var material = new Material(volumeRayDataShader);

        var cb = new CommandBuffer();

        int texId = Shader.PropertyToID("_RayStartPos");
        cb.GetTemporaryRT(texId, -1, -1, 0, FilterMode.Point, RenderTextureFormat.ARGBFloat);
        cb.SetRenderTarget(texId);
        cb.ClearRenderTarget(true, true, Color.clear, 1f);
        cb.DrawRenderer(meshRenderer, material, 0, 0);
        cb.SetGlobalTexture("_RayStartPos", texId);

        texId = Shader.PropertyToID("_RayEndPos");
        cb.GetTemporaryRT(texId, -1, -1, 0, FilterMode.Point, RenderTextureFormat.ARGBFloat);
        cb.SetRenderTarget(texId);
        cb.ClearRenderTarget(true, true, Color.clear, 1f);
        cb.DrawRenderer(meshRenderer, material, 0, 1);
        cb.SetGlobalTexture("_RayEndPos", texId);

        cam.AddCommandBuffer(CameraEvent.BeforeForwardAlpha, cb);
    }
}
