﻿/// Main raytracing shader
Shader "Custom/Volume"
{
	Properties
	{
		_VolumeTex("3D Data Volume Texture", 3D) = "" {}
		_TransferFunction("Transfer function", 2D) = "" {}
		_NumSteps("Num sampling steps", Range(0, 1024)) = 1024
		_Min("Min value", Range(0.0, 1.0)) = 0.0
		_Max("Max value", Range(0.0, 1.0)) = 1.0
		_ShadingIntensity("Shading Intensity", Range(0.0, 1.0)) = 1.0
		_Lerp("Lerp between normal / hue-preserving", Range(0.0, 1.0)) = 1.0
		_Bias("Bias function", Range(0.0, 1.0)) = 0.0
	}
	
	SubShader
	{
		Tags { "Queue" = "Transparent" "RenderType" = "Transparent" }

		Pass
		{
			Cull Front
			ZWrite Off
			Blend SrcAlpha OneMinusSrcAlpha
			//Blend One One

			CGPROGRAM
			#pragma vertex vert
			#pragma fragment frag
			#pragma multi_compile traditional huePreserving

			//#pragma exclude_renderers d3d9
			//#pragma exclude_renderers d3d11_9x

			#include "UnityCG.cginc"
			#include "functions.cginc"

			uniform sampler3D _VolumeTex;
			uniform sampler2D _TransferFunction;
			uniform int _NumSteps;
			uniform float _Min;
			uniform float _Max;
			uniform float _ShadingIntensity;
			uniform float _Lerp;
			uniform float _Bias;

			uniform sampler2D _RayStartPos;
			uniform sampler2D _RayEndPos;

			struct AppData
			{
				float4 position : POSITION;
			};

			struct VertexData
			{
				float4 position : SV_POSITION;
				float3 localPosition : TEXCOORD0;
				float4 screenUv : TEXCOORD1;
			};

			struct Ray 
			{
				float3 origin;
				float3 dir;
				float length;
			};

			VertexData vert(AppData appData)
			{
				VertexData vertexData;
				vertexData.position = UnityObjectToClipPos(appData.position);
				vertexData.localPosition = appData.position;
				vertexData.screenUv = ComputeScreenPos(vertexData.position);

				return vertexData;
			}
			
			float4 frag(VertexData vertexData) : SV_Target
			{
				float2 screenUv = vertexData.screenUv.xy / vertexData.screenUv.w;
				//return float4(tex2D(_RayStartPos, screenUv).rgb, 1.0);
				//return float4(tex2D(_RayDirLength, screenUv));

				Ray ray;
				ray.origin = tex2D(_RayStartPos, screenUv).rgb;

				if (length(ray.origin) == 0.0)
					ray.origin = mul(unity_WorldToObject, float4(_WorldSpaceCameraPos, 1.0)).xyz;

				float3 rayEndPos = tex2D(_RayEndPos, screenUv).rgb;
				float3 rayDir = rayEndPos - ray.origin;

				ray.dir = normalize(rayDir);
				ray.length = length(rayDir);

				bool backToFront = false;
				if (backToFront) // changing direction of ray to perform back to front
				{
					Ray tmp;
					tmp.origin = ray.origin + ray.dir * ray.length;
					tmp.dir = -ray.dir;
					tmp.length = ray.length;
					ray = tmp;
				} 

				float stepSize = 1.732f / _NumSteps;	// 1.732 = max distance inside the box (= the diagonal line)
				float numSteps = floor(ray.length / stepSize);

				float4 finalColorTrad = float4(0, 0, 0, 0);
				float4 finalColorHP = float4(0, 0, 0, 0);

				[loop]
				for (int i = 0; i < numSteps; i++)
				{
					float3 pos = ray.origin + ray.dir * (i * stepSize) + float3(0.5, 0.5, 0.5);

					float value = tex3D(_VolumeTex, pos).r;
					if (value < _Min || value > _Max)
						continue;

					float4 color = tex2D(_TransferFunction, float2(value, 0.5)); //using transfer function to decide color

					//calculating the gradient by comparing neighbour values in each direction. Decrease towards right -> normal points to the right
					float3 gradient = float3(
						tex3D(_VolumeTex, float3(pos.x + stepSize, pos.y, pos.z)).r - value,
						tex3D(_VolumeTex, float3(pos.x, pos.y + stepSize, pos.z)).r - value,
						tex3D(_VolumeTex, float3(pos.x, pos.y, pos.z + stepSize)).r - value
						);

					// transform to world space
					// normalising the gradient (direction of change) 
					float3 normal = -normalize(mul(unity_ObjectToWorld, float4(gradient, 0)).rgb);
					
					// _WorldSpaceLightPos0 = light direction for directional lights
					// _WorldSpaceLightPos0 wasn't defined after removing the skybox, just hardcode the light here
					// float3 L = -normalize(_WorldSpaceLightPos0.xyz);
					float3 L = normalize(float3(-1, 1, -1));
					float NdotL = max(dot(normal, L), 0);
					float3 shaded = color.rgb * 0.1 + color.rgb * NdotL;
					color.rgb = lerp(color.rgb, shaded, _ShadingIntensity);

					if (backToFront) 
					{
						// back to front blending:
						// SRC_ALPHA, ONE_MINUS_SRC_ALPHA

						color.rgb *= color.a;
						finalColorTrad *= (1 - color.a);
						finalColorHP *= (1 - color.a);
						
					} else
					{
						// Front to back blending:
						// GL_ONE_MINUS_DST_ALPHA, GL_ONE
						
						color.rgb *= color.a;
						color *= (1 - finalColorTrad.a);
					}

					finalColorTrad += color;

					finalColorHP = float4(
						huePreservingColorAdd(color, finalColorHP),
						color.a + finalColorHP.a
						);

					if (!backToFront && finalColorTrad.a >= 1.0)
						break;
				}

				// Bias function
				// Use saturation from trad. result to push the gray colors towards the more saturated end.
				float3 finalColorTradHSV = RGBtoHSV(finalColorTrad.rgb);
				float3 finalColorHPHSV = RGBtoHSV(finalColorHP.rgb);
				finalColorHPHSV.y = lerp(finalColorHPHSV.y, finalColorTradHSV.y, _Bias);
				finalColorHP.rgb = HSVtoRGB(finalColorHPHSV);

				return lerp(finalColorTrad, finalColorHP, _Lerp);
			}

			ENDCG
		}
	}
}