/*! \file functions.cginc
 * \brief Functions used in the shader.
 */

/// transforming to hsv to be able to fetch hue
/// from http://lolengine.net/blog/2013/07/27/rgb-to-hsv-in-glsl
/// HSV Ranges are all in [0, 1]
float3 RGBtoHSV(float3 c)
{
	float4 K = float4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
	float4 p = lerp(float4(c.bg, K.wz), float4(c.gb, K.xy), step(c.b, c.g));
	float4 q = lerp(float4(p.xyw, c.r), float4(c.r, p.yzx), step(p.x, c.r));

	float d = q.x - min(q.w, q.y);
	float e = 1.0e-10;
	return float3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

/// transforming back to RGB
/// HSV Ranges are all in [0, 1]
/// from http://lolengine.net/blog/2013/07/27/rgb-to-hsv-in-glsl 
float3 HSVtoRGB(float3 c)
{
	float4 K = float4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
	float3 p = abs(frac(c.xxx + K.xyz) * 6.0 - K.www);
	return float3(c.z * lerp(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y));
}

/// takes opposite color of color1 (rotating hue 180deg), while maintaining saturation and value from color2
/// colors all in rgb
float3 oppositeColor(float3 color1, float3 color2)
{
	float3 color1hsv = RGBtoHSV(color1);
	float3 color2hsv = RGBtoHSV(color2);
	float3 oppositeColor = float3(fmod(((color1hsv.x * 360) + 180), 360) / 360, color2hsv.yz);

	return HSVtoRGB(oppositeColor);
}

/// Checking if the difference between two hues is less than threshold 0.05
bool equalHue(float3 color1, float3 color2)
{
	float3 color1hsv = RGBtoHSV(color1);
	float3 color2hsv = RGBtoHSV(color2);

	if (abs(color1hsv.x - color2hsv.x) < 0.05)
		return true;
	else
		return false;
}

/// Hue preserving blending function from paper
float3 huePreservingColorAdd(float3 color1, float3 color2)
{
	if (equalHue(color1, color2))
		return color1 + color2;

	float3 color2Prime = oppositeColor(color1, color2);
	float3 result = color1 + color2Prime;

	// If color1 was the dominant color, the result should now be the same hue as color1.
	// If this isn't the case, switch colors.
	if (!equalHue(color1, result))
	{
		float3 color1Prime = oppositeColor(color2, color1);
		result = color1Prime + color2;
	}

	return result;
}

/*
float3 huePreservingColorAddBias(float3 color1, float3 color2, float bias)
{
	if (equalHue(color1, color2))
		return color1 + color2;

	float3 color2Prime = oppositeColor(color1, color2);
	float3 result = color1 + color2Prime;

	if (!equalHue(color1, result))
	{
		float3 color1Prime = oppositeColor(color2, color1);
		result = lerp(color1Prime + color2, color2, bias);
	} else {
		result = lerp(color1 + color2Prime, color1, bias);
	}

	return result;
}
*/

/*
float4 blendFrontToBack(float4 src, float4 dest)
{
	// Front to back blending:
	// GL_ONE_MINUS_DST_ALPHA, GL_ONE

	//finalColor.rgb = finalColor.rgb + (1.0 - finalColor.a) * color.rgb;
	//finalColor.a = finalColor.a + (1.0 - finalColor.a) * color.a;

	src.rgb *= src.a;
	dest = (1.0 - dest.a) * src + dest;

	return dest;
}

float4 blendBackToFront(float4 src, float4 dest)
{
	src.rgb *= src.a;
	dest = src + (1.0 - src.a) * dest;

	//dest = src.a * src + (1.0 - src.a) * dest;

	return dest;
}
*/